// protect.cpp:
//
// scan a disk and change protections on executable files
//
// jim frost 02.16.96

#include <windows.h>
#include <stdio.h>
#include "security.hpp"

unsigned int Verbose;

char* ExecutablePatterns[] = {
  "*.exe",
  "*.com",
  "*.dll",
  "*.bat",
  NULL
};

SecurityDescriptor* ProtectedDescriptor;

void ProtectFile(const char* path)
{
  if (!SetFileSecurityDescriptor(path, (OWNER_SECURITY_INFORMATION |
					GROUP_SECURITY_INFORMATION |
					DACL_SECURITY_INFORMATION),
				 ProtectedDescriptor))
    fprintf(stderr, "%s: %s\n", path, strerror(GetLastError()));
  else
    printf("Protected %s\n", path);
}

void ProtectDirectory(const char* dir_name)
{
  char path[MAX_PATH];
  HANDLE dir_handle;
  WIN32_FIND_DATA entry;

  for (int pattern = 0; ExecutablePatterns[pattern] != NULL; pattern++) {
    sprintf(path, "%s\\%s", dir_name, ExecutablePatterns[pattern]);
    dir_handle = FindFirstFile(path, &entry);
    if (dir_handle == INVALID_HANDLE_VALUE)
      continue;
    sprintf(path, "%s\\", dir_name);
    char* fn_start = path + strlen(path);
    do {
      strcpy(fn_start, entry.cFileName);
      ProtectFile(path);
    } while (FindNextFile(dir_handle, &entry) == TRUE);
    FindClose(dir_handle);
  }

  // walk complete directory and recurse on subdirectories
  sprintf(path, "%s\\*", dir_name);
  dir_handle = FindFirstFile(path, &entry);
  if (dir_handle == INVALID_HANDLE_VALUE)
    return;
  do {
    if (entry.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) {
      // skip . and .. entries
      if (!strcmp(entry.cFileName, ".") || !strcmp(entry.cFileName, ".."))
	continue;
      sprintf(path, "%s\\%s", dir_name, entry.cFileName);
      ProtectDirectory(path);
    }
  } while (FindNextFile(dir_handle, &entry));
  FindClose(dir_handle);
}

// build a descriptor with very limited access characteristics
void
InitDescriptor(void)
{
  SecurityIdentifier* admin  = CreateSecurityIdentifier("Administrator");
  SecurityIdentifier* admin_group = CreateSecurityIdentifier("Administrators");
  SecurityIdentifier* everyone = CreateSecurityIdentifier("Everyone");
  if (admin == NULL) {
    fprintf(stderr, "No administrator account?  Sorry, can't continue!\n");
    exit(1);
  }
  if (admin_group == NULL) {
    fprintf(stderr, "No administrators group?  Sorry, can't continue!\n");
    exit(1);
  }
  if (everyone == NULL) {
    fprintf(stderr, "No 'everyone' group?  Sorry, can't continue!\n");
    exit(1);
  }

  AccessControlList* acl = new AccessControlList;

  acl->AppendAllowAce(0, admin_group, FILE_ALL_ACCESS);
  acl->AppendAllowAce(0, everyone, FILE_GENERIC_READ|FILE_GENERIC_EXECUTE);

  ProtectedDescriptor = new SecurityDescriptor;
  ProtectedDescriptor->SetOwner(admin);
  ProtectedDescriptor->SetDiscretionaryAcl(acl);
}

void protect(LPCSTR fileName)
{
  InitDescriptor();

  if (fileName == NULL) 
  {
    ProtectDirectory(".");
  }
  else 
  { 
      ProtectDirectory(fileName);
  }
}
